'use strict';

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _sinonChai = require('sinon-chai');

var _sinonChai2 = _interopRequireDefault(_sinonChai);

var _index = require('../index');

var _walk = require('../walk');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

_chai2.default.use(_sinonChai2.default);
var expect = _chai2.default.expect;

var Node = function Node(data) {
    var children = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];

    _classCallCheck(this, Node);

    this.data = data;
    this.children = children;
};

function getChildren(node) {
    return node.children;
}

function makeNode(node, children) {
    return new Node(node.data, children);
}

function isBranch(node) {
    return !!getChildren(node).length;
}

var log = function log(node) {
    console.log('LOG NODE:', node.data);
    return node;
};

describe('TreeZipper', function () {
    function getTree() {
        return new Node(1, [new Node(2, [new Node(4), new Node(5)]), new Node(3)]);
    }

    var Zipper = (0, _index.makeZipper)(isBranch, getChildren, makeNode);

    function incrementNode(node) {
        return new Node(node.data + 1, node.children);
    }

    it('postwalk works', function () {
        var z = Zipper.from(getTree());

        function gatherNumbers(listRef, node) {
            listRef.push(node.data);
            return node;
        }

        // Assert initial is correct
        var initialNumbers = [];
        (0, _walk.preWalk)(gatherNumbers.bind(null, initialNumbers), z);
        expect(initialNumbers).to.deep.equal([1, 2, 4, 5, 3]);

        // Do the incrementation.
        z = (0, _walk.postWalk)(incrementNode, z);

        // Check result.
        var afterNumbers = [];
        (0, _walk.preWalk)(gatherNumbers.bind(null, afterNumbers), z);
        expect(afterNumbers).to.deep.equal([2, 3, 5, 6, 4]);
    });
});